<?php
/**
 * Plugin Name: WooCommerce Category Product Thumbnails
 * Plugin URI: http://www.gazchap.com/posts/woocommerce-category-product-thumbnails/
 * Description: Automatically use a product thumbnail as a category thumbnail if no category thumbnail is set
 * Author: Gareth 'GazChap' Griffiths
 * Version: 1.0
 * Author URI: http://www.gazchap.com/
 */

namespace GazChap;

class WC_Category_Product_Thumbnails {

	private $shuffle = true;
	private $recurse_category_ids = true;
	private $limit = 20; // number of most recent products to fetch when shuffle is true, lower means faster

	/**
	 * WC_Category_Product_Thumbnails constructor.
	 * Sets a default limit (to -1, i.e. all posts) if none already set and then replaces WC actions with ours
	 */
	public function __construct() {
		add_action( 'plugins_loaded', array( $this, 'replace_wc_actions' ) );

		if ( !$this->limit ) $this->limit = -1;
	}

	/**
	 * Removes the action that puts the thumbnail before the subcategory title, and replaces it with our version
	 */
	public function replace_wc_actions() {
		remove_action( 'woocommerce_before_subcategory_title', 'woocommerce_subcategory_thumbnail', 10 );
		add_action( 'woocommerce_before_subcategory_title', array( $this, 'auto_subcategory_thumbnail' ) );
	}

	/**
	 * The function that does all the donkey work.
	 * @param \WP_Term $category - the category that we're dealing with
	 */
	public function auto_subcategory_thumbnail( $category ) {

		// does this category already have a thumbnail defined? if so, use that instead
		if ( get_term_meta( $category->term_id, 'thumbnail_id', true ) ) {
			woocommerce_subcategory_thumbnail( $category );
			return;
		}

		// get a list of category IDs inside this category (so we're fetching products from all subcategories, not just the top level one)
		if ( $this->recurse_category_ids ) {
			$category_ids = $this->get_sub_category_ids( $category );
		} else {
			$category_ids = array( $category->term_id );
		}

		$query_args = array(
			'posts_per_page' => $this->shuffle ? $this->limit : 1,
			'post_status' => 'publish',
			'post_type' => 'product',
			'meta_query' => array(
				array(
					'key' => '_thumbnail_id',
					'value' => '',
					'compare' => '!=',
				),
			),
			'tax_query' => array(
				array(
					'taxonomy' => 'product_cat',
					'field' => 'term_id',
					'terms' => $category_ids,
					'operator' => 'IN',
				),
			),
		);

		$products = get_posts( $query_args );
		if ( $products ) {
			echo get_the_post_thumbnail( $products[ array_rand( $products ) ]->ID, 'shop_thumbnail' );
		} else {
			// show the default placeholder category image if there's no products inside this one
			woocommerce_subcategory_thumbnail( $category );
		}
	}

	/**
	 * Recursive function to fetch a list of child category IDs for the one passed
	 *
	 * @param \WP_Term $start - the category to start from
	 * @param array $results - this just stores the results as they're being built up
	 *
	 * @return array - an array of term IDs for each product_cat inside the original one
	 */
	private function get_sub_category_ids( $start, $results = array() ) {
		if ( !is_array( $results ) ) $results = array();

		$results[] = $start->term_id;
		$cats = get_terms( array( 'taxonomy' => 'product_cat', 'hide_empty' => false, 'parent' => $start->term_id ) );
		if ( is_array( $cats ) ) {
			foreach( $cats as $cat ) {
				$results = $this->get_sub_category_ids( $cat, $results );
			}
		}

		return $results;
	}
}

new WC_Category_Product_Thumbnails();